package com.ekexiu.portal.oauth;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.method.SetCookie;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.state.StateCode;

import com.ekexiu.portal.dao.ProfessorDao;
import com.ekexiu.portal.dao.UserDao;
import com.ekexiu.portal.dao.UserOpenIdDao;
import com.ekexiu.portal.po.User;
import com.ekexiu.portal.po.UserOpenId;
import com.ekexiu.portal.pojo.SessionUser;
import com.ekexiu.portal.service.SysService;

@Path("/oauth")
public class OAuthService {

    private long expriesWithOpenId = 1000 * 60 * 10;

    @Autowrie
    private UserOpenIdDao userOpenIdDao;
    @Autowrie
    private UserDao userDao;
    @Autowrie
    private ProfessorDao professorDao;

    private Map<String, OAuthHandler> handlers;

    public Map<String, OAuthHandler> getHandlers() {
        return handlers;
    }

    public void setHandlers(Map<String, OAuthHandler> handlers) {
        this.handlers = handlers;
    }

    public UserOpenIdDao getUserOpenIdDao() {
        return userOpenIdDao;
    }

    public void setUserOpenIdDao(UserOpenIdDao userOpenIdDao) {
        this.userOpenIdDao = userOpenIdDao;
    }

    public UserDao getUserDao() {
        return userDao;
    }

    public void setUserDao(UserDao userDao) {
        this.userDao = userDao;
    }

    public long getExpriesWithOpenId() {
        return expriesWithOpenId;
    }

    public void setExpriesWithOpenId(long expriesWithOpenId) {
        this.expriesWithOpenId = expriesWithOpenId;
    }

    public ProfessorDao getProfessorDao() {
        return professorDao;
    }

    public void setProfessorDao(ProfessorDao professorDao) {
        this.professorDao = professorDao;
    }

    @Get
    @Path("/validCode")
    public AuthLoginResponse validCode(@JdbcConn Connection con, String code, String state) throws SQLException, JfwBaseException {
        OAuthHandler oah = this.handlers.get(state);
        if (oah == null)
            throw new IllegalArgumentException("非法的参数{state=" + state);
        OAuthUser ou = oah.login(code);
        UserOpenId uoi = this.userOpenIdDao.query(con, oah.getType(), ou.getOpenId());

        StateCode<OAuthUser, UserOpenId> sc = new StateCode<OAuthUser, UserOpenId>();
        final String key = JfwAppContext.cacheObjectAndGenKey(sc);
        sc.setKey(ou);
        sc.setValue(uoi);
        sc.setBuildTime(System.currentTimeMillis());
        sc.setExpiredTime(sc.getBuildTime() + this.expriesWithOpenId);
        JfwAppContext.getScheduledExecutorService().schedule(new Runnable() {
            @Override
            public void run() {
                JfwAppContext.removeCachedObject(key);
            }
        }, this.expriesWithOpenId, TimeUnit.MILLISECONDS);
        AuthLoginResponse alr = new AuthLoginResponse();
        alr.setAssociated(null != uoi);
        alr.setAuthCode(key);
        return alr;
    }

    @SetCookie(checkResultNull = true, path = "/", value = { "userid=result.getId()", "userMobilePhone=result.getMobilePhone()", "userType=result.getType()",
            "userAuth=String.valueOf(result.isAuth())", "userEmail=result.getEmail()==null?\"\":result.getEmail()",
            "userName=result.getName()==null?\"\":java.net.URLEncoder.encode(result.getName(),\"utf-8\")" })
    @Post
    @Path("/login")
    public SessionUser login(@JdbcConn Connection con, String authCode) throws SQLException, JfwBaseException {
        @SuppressWarnings("unchecked")
        StateCode<OAuthUser, UserOpenId> sc = (StateCode<OAuthUser, UserOpenId>) JfwAppContext.getCachedObject(authCode);
        JfwAppContext.removeCachedObject(authCode);
        if (sc == null || sc.getExpiredTime() < System.currentTimeMillis())
            throw new JfwBaseException(-1, "authCode is expired");
        if (null == sc.getValue())
            throw new JfwBaseException(-2, "authCode is invalid openid not associate local user");

        User user = this.userDao.query(con, sc.getValue().getUserid());
        if (null == user)
            return null;
        SessionUser ret = new SessionUser();
        ret.setId(user.getId());
        if (null != this.professorDao.query(con, user.getId())) {
            ret.setName(this.professorDao.query(con, user.getId()).getName());
        }
        ret.setMobilePhone(user.getMobilePhone());
        ret.setType(user.getUserType());
        ret.setEmail(user.getEmail());
        ret.setAuth(!SysService.DEFAULT_PASS_WORD.equals(user.getPasswd()));
        return ret;
    }

    @Post
    @Path("/validOpenid")
    public AuthLoginResponse validOpenid(@JdbcConn Connection con, String authType, String openid) throws SQLException, JfwBaseException {
        OAuthUser ou = new OAuthUser();
        ou.setType(authType);
        ou.setOpenId(openid);
        UserOpenId uoi = this.userOpenIdDao.query(con, authType, openid);
        AuthLoginResponse alr = new AuthLoginResponse();
        StateCode<OAuthUser, UserOpenId> sc = new StateCode<OAuthUser, UserOpenId>();
        final String key = JfwAppContext.cacheObjectAndGenKey(sc);
        sc.setKey(ou);
        sc.setValue(uoi);
        sc.setBuildTime(System.currentTimeMillis());
        sc.setExpiredTime(sc.getBuildTime() + this.expriesWithOpenId);
        JfwAppContext.getScheduledExecutorService().schedule(new Runnable() {
            @Override
            public void run() {
                JfwAppContext.removeCachedObject(key);
            }
        }, this.expriesWithOpenId, TimeUnit.MILLISECONDS);
        alr.setAssociated(null != uoi);
        alr.setAuthCode(key);
        return alr;
    }

    

    @Get
    @Path("/redirectUris")
    public Map<String, String> getRedirectUris() {
        Map<String, String> ret = new HashMap<String, String>();
        for (Map.Entry<String, OAuthHandler> h : this.handlers.entrySet()) {
            ret.put(h.getKey(), h.getValue().getLoginUrl());
        }
        return ret;
    }

    public static class AuthLoginResponse {
        private boolean associated;
        private String authCode;

        public boolean isAssociated() {
            return associated;
        }

        public void setAssociated(boolean associated) {
            this.associated = associated;
        }

        public String getAuthCode() {
            return authCode;
        }

        public void setAuthCode(String authCode) {
            this.authCode = authCode;
        }
    }

}
